-- ============================================================= --
-- KEYBOARD STEERING MOD
-- ============================================================= --
KeyboardSteering = {};

addModEventListener(KeyboardSteering);

function KeyboardSteering:onUpdate(superFunc, dt, isActiveForInput, isActiveForInputIgnoreSelection, isSelected)
	--print("Modified Steering Function")
    local spec = self.spec_drivable
    -- update inputs on client side for basic controls
    if self.isClient then
        if self.getIsEntered ~= nil and self:getIsEntered() then
            if self:getIsActiveForInput(true, true) then
                if self:getIsVehicleControlledByPlayer() then
                    spec.doHandbrake = false
                    -- gas and brake pedal
                    local axisForward = MathUtil.clamp((spec.lastInputValues.axisAccelerate - spec.lastInputValues.axisBrake), -1, 1)
                    spec.axisForward = axisForward
                    -- steering
                    local speedFactor = 1.0
                    local sensitivitySetting = g_gameSettings:getValue(GameSettings.SETTING.STEERING_SENSITIVITY)
					
                    if spec.lastInputValues.axisSteerIsAnalog then
                        local isArticulatedSteering = self.spec_articulatedAxis ~= nil and self.spec_articulatedAxis.componentJoint ~= nil
                        if isArticulatedSteering then
                            speedFactor = 1.5
                        else
                            speedFactor = 2.5
                        end
                        -- only use steering speed for gamepads
                        if spec.lastInputValues.axisSteerDeviceCategory == InputDevice.CATEGORY.GAMEPAD then
                            speedFactor = speedFactor * sensitivitySetting
                        end
                    else
                        if spec.lastInputValues.axisSteer == 0 then
                            local rotateBackSpeedSetting = g_gameSettings:getValue(GameSettings.SETTING.STEERING_BACK_SPEED) / 10
                            if self.speedDependentRotateBack then
                                local speed = self:getLastSpeed()
                                local setting = rotateBackSpeedSetting / 0.5
								local maxSpeed = 50 --self:getMotor():getMaximumForwardSpeed()*3.6
								local roadSpeedFactor = math.min(speed/maxSpeed, 1.0)
								local steeringAngleFactor = math.abs(spec.axisSide)
								speedFactor = speedFactor * setting * roadSpeedFactor * steeringAngleFactor
                            end
                            speedFactor = speedFactor * (self.autoRotateBackSpeed or 1) / 1.5
                        else
							local keyPressTimeFactor = KeyboardSteering.timeKeyHeld/KeyboardSteering.maxKeyHeldTime
                            speedFactor = math.min(1.0/(self.lastSpeed*spec.speedRotScale+spec.speedRotScaleOffset), 1);
                            speedFactor = speedFactor * sensitivitySetting * keyPressTimeFactor
                        end
                    end
                    local steeringDuration = (self.wheelSteeringDuration or 1) * 1000
                    local rotDelta = (dt / steeringDuration) * speedFactor
					
					-- REMOVE ALWAYS TURNING RIGHT BEHAVIOUR WHEN HOLDING DOWN BOTH KEYS
					if KeyboardSteering.goStraight then
						--print("go straight")
						if math.abs(spec.axisSide)<0.005 then
							spec.axisSide = 0
							rotDelta = 0
						else
							if spec.axisSide>0.0 then
								if spec.axisSide==1.0 then
									spec.axisSide=spec.axisSide-rotDelta
								end
								rotDelta = -rotDelta
							end
						end
					end
					
					if spec.lastInputValues.axisSteer > spec.axisSide then
                        spec.axisSide = math.min(spec.lastInputValues.axisSteer, spec.axisSide + rotDelta)
                    elseif spec.lastInputValues.axisSteer < spec.axisSide then
                        spec.axisSide = math.max(spec.lastInputValues.axisSteer, spec.axisSide - rotDelta)
                    end
					--Limit steering angle depending on road speed
					local speed = self:getLastSpeed()
					local rotateBackSetting = g_gameSettings:getValue(GameSettings.SETTING.STEERING_BACK_SPEED) / 10
					local minSpeed = 50 - (30*rotateBackSetting)
					local maxSpeed = 500 - (360*rotateBackSetting)
					if speed>minSpeed then
						local maxAngle = MathUtil.clamp(1.0 - (speed-minSpeed)/(maxSpeed-minSpeed), 0.2, 1)
						spec.axisSide = MathUtil.clamp(spec.axisSide, -maxAngle, maxAngle)
					end
                else
                    spec.axisForward = 0
                    if self.rotatedTime < 0 then
                        spec.axisSide = self.rotatedTime / -self.maxRotTime / self:getSteeringDirection()
                    else
                        spec.axisSide = self.rotatedTime / self.minRotTime / self:getSteeringDirection()
                    end
                end
            else
                spec.doHandbrake = true
                spec.axisForward = 0
            end
            -- prepare for next frame
            spec.lastInputValues.axisAccelerate = 0
            spec.lastInputValues.axisBrake = 0
            spec.lastInputValues.axisSteer = 0
            -- prepare network update
            if spec.axisForward ~= spec.axisForwardSend or spec.axisSide ~= spec.axisSideSend or spec.doHandbrake ~= spec.doHandbrakeSend then
                spec.axisForwardSend = spec.axisForward
                spec.axisSideSend = spec.axisSide
                spec.doHandbrakeSend = spec.doHandbrake
                self:raiseDirtyFlags(spec.dirtyFlag)
            end
        end
    end
    -- update inputs on client side for cruise control
    if self.isClient then
        if self.getIsEntered ~= nil and self:getIsEntered() then
            -- cruise control state
            local inputValue = spec.lastInputValues.cruiseControlState
            spec.lastInputValues.cruiseControlState = 0
            if inputValue == 1 then
                if spec.cruiseControl.topSpeedTime == Drivable.CRUISECONTROL_FULL_TOGGLE_TIME then
                    if spec.cruiseControl.state == Drivable.CRUISECONTROL_STATE_OFF then
                        self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE)
                    else
                        self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF)
                    end
                end
                if spec.cruiseControl.topSpeedTime > 0 then
                    spec.cruiseControl.topSpeedTime = spec.cruiseControl.topSpeedTime - dt
                    if spec.cruiseControl.topSpeedTime < 0 then
                        self:setCruiseControlState(Drivable.CRUISECONTROL_STATE_FULL)
                    end
                end
            else
                spec.cruiseControl.topSpeedTime = Drivable.CRUISECONTROL_FULL_TOGGLE_TIME
            end
            -- cruise control value
            local lastCruiseControlValue = spec.lastInputValues.cruiseControlValue
            spec.lastInputValues.cruiseControlValue = 0
            if lastCruiseControlValue ~= 0 then
                spec.cruiseControl.changeCurrentDelay = spec.cruiseControl.changeCurrentDelay - (dt * spec.cruiseControl.changeMultiplier)
                spec.cruiseControl.changeMultiplier = math.min(spec.cruiseControl.changeMultiplier + (dt * 0.003), 10)
                if spec.cruiseControl.changeCurrentDelay < 0 then
                    spec.cruiseControl.changeCurrentDelay = spec.cruiseControl.changeDelay
                    local dir = MathUtil.sign(lastCruiseControlValue)
                    local speed = spec.cruiseControl.speed + dir
                    self:setCruiseControlMaxSpeed(speed)
                    if spec.cruiseControl.speed ~= spec.cruiseControl.speedSent then
                        if g_server ~= nil then
                            g_server:broadcastEvent(SetCruiseControlSpeedEvent:new(self, spec.cruiseControl.speed), nil, nil, self)
                        else
                            g_client:getServerConnection():sendEvent(SetCruiseControlSpeedEvent:new(self, spec.cruiseControl.speed))
                        end
                        spec.cruiseControl.speedSent = spec.cruiseControl.speed
                    end
                end
            else
                spec.cruiseControl.changeCurrentDelay = 0
                spec.cruiseControl.changeMultiplier = 1
            end
        end
    end
    local isControlled = self.getIsControlled ~= nil and self:getIsControlled()
    -- update vehicle physics on server side
    if self:getIsVehicleControlledByPlayer() then
        if self.isServer then
            if isControlled then
                -- lock max speed to working tool
                local speed,_ = self:getSpeedLimit(true)
                if spec.cruiseControl.state == Drivable.CRUISECONTROL_STATE_ACTIVE then
                    speed = math.min(speed, spec.cruiseControl.speed)
                end
                self:getMotor():setSpeedLimit(speed)
                self:updateVehiclePhysics(spec.axisForward, spec.axisSide, spec.doHandbrake, dt)
            end
        end
    end
    -- just a visual update of the steering wheel
    if self.isClient and isControlled then
        self:updateSteeringWheel(spec.steeringWheel, dt, 1)
    end
end

function KeyboardSteering:loadMap(name)
	--print("Load Mod: 'Keyboard Steering'")
	Drivable.onUpdate = Utils.overwrittenFunction(Drivable.onUpdate, KeyboardSteering.onUpdate)

	self.turnLeft			= false		-- input turn direction (via key press)
	self.turnRight			= false		-- input turn direction (via key press)
	self.timeKeyHeld		= 0			-- key down time (via key press)
	self.maxKeyHeldTime		= 500		-- key down time (ms)
	self.goStraight			= false		-- flag to go straight after pressing A and D
end

function KeyboardSteering:deleteMap()
end

function KeyboardSteering:mouseEvent(posX, posY, isDown, isUp, button)
end

function KeyboardSteering:keyEvent(unicode, sym, modifier, isDown)
	--InputAction.AXIS_MOVE_SIDE_PLAYER
	if sym == 97 then
		if isDown then
			--print("TURN LEFT")
			self.turnLeft = true
		else
			--print("NO TURN")
			self.turnLeft = false
		end
	end
	if sym == 100 then
		if isDown then
			--print("TURN RIGHT")
			self.turnRight = true
		else
			--print("NO TURN")
			self.turnRight = false
		end
	end
end

function KeyboardSteering:draw()
end

function KeyboardSteering:update(dt)
	--UPDATE KEY PRESS TIMES
	if self.turnLeft or self.turnRight then
		if self.timeKeyHeld < self.maxKeyHeldTime then
			self.timeKeyHeld = self.timeKeyHeld + dt
		else
			self.timeKeyHeld = self.maxKeyHeldTime
		end
		if self.timeKeyHeld > self.maxKeyHeldTime/10 then
			self.goStraight = false
		end
	else
		if self.timeKeyHeld-dt > 0 then
			self.timeKeyHeld = self.timeKeyHeld - dt
		else
			self.timeKeyHeld = 0
		end
	end
	
	if self.turnLeft and self.turnRight then
		--HOLDING DOWN BOTH KEYS
		KeyboardSteering.goStraight = true
	end	
end